/* 
 *   ccglue - cscope,ctags glue: Process ctags and cscope output to
 *        produce cross-reference table (for use with Vim CCTree plugin)
 *        and more ...
 *   Copyright (C) April, 2011,  Hari Rangarajan 
 *   Contact: hariranga@users.sourceforge.net

 *   "ccglue" is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with "ccglue".  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "seqfile.h"
#include "typedefs.h"
#include "sym_mgr.h"
#include "cscoperdr.h"

typedef void (*cscope_db_line_process_func) (cscope_db_rdr_t *, 
        sym_table_t *, uchar*);

RC_t process_cscope_files_using_func (sym_table_t* sym_table, uchar* cscope_files,
        cscope_db_line_process_func func)
{
    seq_file_t	*seqfile;
    uchar		 line[1024];
    int			 idx = 0;
    cscope_db_rdr_t *rdr;
    int lines = 0;
    char *cscope_file;
    uchar  cscope_files_var[512];

    cscope_db_rdr_create(&rdr);

    /* need to copy before we try tokenizing */
    strcpy(cscope_files_var, cscope_files);

    cscope_file = strtok(cscope_files_var, ",");
    while (cscope_file != NULL) {
        if (seq_file_reader_create(&seqfile, cscope_file) == RC_SUCCESS) {
            while (seq_file_readline(seqfile, line, sizeof(line)) != RC_FAILURE) {
                lines++;
                (*func)(rdr, sym_table, line);
            }
            seq_file_destroy(seqfile);
        } else {
            return RC_FAILURE;
        }
        cscope_file = strtok(NULL, ",");
    }
    cscope_db_rdr_destroy(rdr);

    return RC_SUCCESS;
}
    

void process_cscope_files_to_build_sym_table (sym_table_t* sym_table, 
        uchar* cscope_files)
{
    process_cscope_files_using_func(sym_table, cscope_files, 
            cscope_db_rdr_build_sym_table);
}

void process_cscope_files_to_build_xrefs (sym_table_t* sym_table,
        uchar* cscope_files)
{
    process_cscope_files_using_func(sym_table, cscope_files, 
           cscope_db_rdr_build_xref);
}

