/* 
 *   ccglue - cscope,ctags glue: Process ctags and cscope output to
 *        produce cross-reference table (for use with Vim CCTree plugin)
 *        and more ...
 *   Copyright (C) April, 2011,  Hari Rangarajan 
 *   Contact: hariranga@users.sourceforge.net

 *   "ccglue" is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with "ccglue".  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "cscoperdr.h"
#include <stdio.h>
#include <string.h>

struct cscope_db_rdr {
    sym_entry_t	*in_func;
    sym_entry_t	*in_file;
    sym_entry_t	*in_macro;
    sym_entry_t	*in_enum;
};


RC_t cscope_db_rdr_create (cscope_db_rdr_t **rdr)
{
    *rdr = (cscope_db_rdr_t *) malloc(sizeof(cscope_db_rdr_t));
    memset(*rdr, 0, sizeof(cscope_db_rdr_t));
    if (*rdr == NULL) {
        return RC_FAILURE;
    }

    return RC_SUCCESS;
}

void cscope_db_rdr_destroy (cscope_db_rdr_t *rdr)
{
    free(rdr);
}

void cscope_db_rdr_build_xref (cscope_db_rdr_t *rdr,
        sym_table_t *sym_table, uchar* line)
{
    uchar	 temp[1024];
    sym_entry_t *ref_func;
    sym_entry_t *ref_file;
    sym_entry_t *untagged_sym;

    if (line[0] !='\t') {
        sprintf(temp, "%s", &line[0]);
        untagged_sym = sym_table_lookup(sym_table, temp);
        if (untagged_sym == NULL) {
            return;
        }
        if (rdr->in_macro) {
            sym_table_mark_xref(rdr->in_macro, untagged_sym);
        } else if (rdr->in_func) {
            sym_table_mark_xref(rdr->in_func, untagged_sym);
        }
    } else if (line[0] == '\t') {
        sprintf(temp, "%s", &line[2]);
        if (rdr->in_macro) {
            switch (line[1]) {
            case '`':
                ref_func = sym_table_add_ext(sym_table, temp);
                if (rdr->in_macro && ref_func) {
                    sym_table_mark_xref(rdr->in_macro, ref_func);
                }
                break;
            case ')':
                rdr->in_macro = NULL;
            default:
                break;
            }
        } else if (rdr->in_func) {
            switch (line[1]) {
            case '`':
                ref_func = sym_table_add_ext(sym_table, temp);
                if (rdr->in_func && ref_func) {
                    sym_table_mark_xref(rdr->in_func, ref_func);
                }
                break;
            case '}':
                rdr->in_func = NULL;
                break;
            case '#':
                rdr->in_macro = sym_table_lookup(sym_table, temp);
                break;
            default:
                break;
            }
        } else {
            switch (line[1]) {
            case '$': 
                rdr->in_func = sym_table_lookup(sym_table, temp);
                break;
            case '#': 
                rdr->in_macro = sym_table_lookup(sym_table, temp);
                break;
            case '~':
                ref_file = sym_table_add_ext(sym_table, &temp[1]);
                if (ref_file && rdr->in_file) {
                    sym_table_mark_xref(rdr->in_file, ref_file);
                }
                break;
            case '@':
                if (temp[0] != '\0') {
                    rdr->in_file = sym_table_lookup(sym_table, temp);
                }
                break;
            default:
                break;
            }
        }
    }
}


void cscope_db_rdr_build_sym_table (cscope_db_rdr_t *rdr,
        sym_table_t *sym_table, uchar* line)
{
    uchar        temp[1024];
    sym_entry_t *ref_func;
    sym_entry_t *ref_file;

    if (line[0] =='\t') {
        sprintf(temp, "%s", &line[2]);
        if (rdr->in_enum) {
            switch (line[1]) {
            case 'm':
                (void) sym_table_add_ext(sym_table, temp);
                return;
            default:
                rdr->in_enum = NULL;
                break;
            } 
        }
        /* fall-through path or otherwise */
        switch (line[1]) {
        case '$': 
            (void) sym_table_add_ext(sym_table, temp);
            break;
        case '#': 
            (void) sym_table_add_ext(sym_table, temp);
            break;
        case '@':
            if (temp[0] != '\0') {
                (void) sym_table_add_ext(sym_table, temp);
            }
            break;
        case 'e':
            rdr->in_enum = sym_table_add_ext(sym_table, temp);
            break;
        case 'g':
            (void) sym_table_add_ext(sym_table, temp);
            break;
#if 0
            /* typedefs are not that useful to track */
        case 't':
            (void) sym_table_add_ext(sym_table, temp);
            break;
#endif
        default:
            break;
        }
    }
}

