/* 
 *   ccglue - cscope,ctags glue: Process ctags and cscope output to
 *        produce cross-reference table (for use with Vim CCTree plugin)
 *        and more ...
 *   Copyright (C) April, 2011,  Hari Rangarajan 
 *   Contact: hariranga@users.sourceforge.net

 *   "ccglue" is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with "ccglue".  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*  ccglue 
 * 
 *
 *  Released under the terms of the GNU GPL v2.0 */



#include "typedefs.h"
#include "digraph.h"
#include <string.h>

struct digraph_map {
	uchar	table[128][2];
};

struct digraph_compress_map {
	uchar rev_index1[128];
	uchar rev_index2[128];
	uchar table[12][12];
};

void digraph_build_char_compress_map (digraph_compress_map_t* map, 
											uchar* seq1, uchar* seq2)
{
	int i,j, idx = 128;
	
	memset(map, -1, sizeof(digraph_compress_map_t));
	/* build reverse indexes */
	for(i=0; i<strlen(seq1); i++) {
		map->rev_index1[seq1[i]] = i;
	}
	for(i=0; i<strlen(seq2); i++) {
		map->rev_index2[seq2[i]] = i;
	}
	for(i=0; i<strlen(seq1); i++) {
			for (j=0; j<strlen(seq2); j++) {
				map->table[i][j] =  idx++;
			}
	}
}

static uchar* numseq1 = ",0123456789";
static uchar* numseq2 = ",0123456789";

digraph_compress_map_t* digraph_get_numeric_map ()
{
	static digraph_compress_map_t map;
	static int i = 0;

	if (i == 0) {
			digraph_build_char_compress_map(&map, numseq1, numseq2);
			i = 1;
	}
	return &map;
}

int digraph_compress_string_with_map (uchar** uncmpstr, uchar* cmpstr, int cmpstr_maxlen, 
                    digraph_compress_map_t* map)
{
	int idx;
    int cmpstr_len = 0;
    int rc = 0;             /* finished the entire string */

	while (**uncmpstr != '\0') {
		if (map->rev_index1[**uncmpstr] != 0xFF) {
			if ((*uncmpstr)[1] != '\0' && map->rev_index2[(*uncmpstr)[1]] != 0xFF) {
				*cmpstr = map->table[map->rev_index1[**uncmpstr]][map->rev_index2[(*uncmpstr)[1]]];
				cmpstr++;
				*uncmpstr += 2;
			} else {
				*cmpstr = **uncmpstr;
				cmpstr++;
				(*uncmpstr)++;
			}
		} else {
			*cmpstr = **uncmpstr;
			cmpstr++;
			(*uncmpstr)++;
		}
        cmpstr_len++;
        if (cmpstr_len == cmpstr_maxlen) {
            rc = 1; /* more to come */
            break;
        }
	}
	*cmpstr = '\0';
    return rc;
}



static uchar* charseq1 = " teisaprnl(of)=c";
static uchar* charseq2 = " tnerpla";

void digraph_build_char_uncompress_map (digraph_map_t* map, uchar* seq1, uchar* seq2)
{
	int i,j, idx = 0;
	char idxchar;

	for(i=0; i<strlen(seq1); i++) {
		for (j=0; j<strlen(seq2); j++) {
			map->table[idx][0] = seq1[i];
			map->table[idx][1] = seq2[j];
			idx++;
		}
	}
}


digraph_map_t* digraph_get_letter_map ()
{
	static digraph_map_t map;
	static int i = 0;

	if (i == 0) {
			digraph_build_char_uncompress_map(&map, charseq1, charseq2);
			i = 1;
	}
	return &map;
}

void digraph_uncompress_string_with_map (uchar* cmpstr, uchar* uncmpstr, digraph_map_t* map)
{
	int idx;

	while (*cmpstr != '\0') {
		idx = (uchar) (*cmpstr) - 128;
		if (idx > 0) {
			uncmpstr[0] = map->table[idx][0];
			uncmpstr[1] = map->table[idx][1];
			uncmpstr += 2*sizeof(uchar);
		} else {
			*uncmpstr = *cmpstr;
			uncmpstr++;
		}
		cmpstr++;
	}
	*uncmpstr = '\0';
}

int digraph_compress_string_with_map_write_to_seqfile (uchar* uncmpstr, 
                    seq_file_t *seq_file, digraph_compress_map_t* map)
{
    int idx;
    int rc = 0;             /* finished the entire string */
    uchar cmpstr;

    while (*uncmpstr != '\0') {
        if (map->rev_index1[*uncmpstr] != 0xFF) {
            if ((uncmpstr)[1] != '\0' && map->rev_index2[(uncmpstr)[1]] != 0xFF) {
                cmpstr = map->table[map->rev_index1[*uncmpstr]]
                                    [map->rev_index2[(uncmpstr)[1]]];
                uncmpstr += 2;
            } else {
                cmpstr = *uncmpstr;
                (uncmpstr)++;
            }
        } else {
            cmpstr = *uncmpstr;
            (uncmpstr)++;
        }
        seq_file_write_char(seq_file, cmpstr);
    }
    return rc;
}


void digraph_uncompress_string_with_map_write_to_seqfile (uchar* cmpstr, 
        seq_file_t  *seq_file, digraph_map_t* map)
{
	int idx;

	while (*cmpstr != '\0') {
		idx = (uchar) (*cmpstr) - 128;
		if (idx > 0) {
			seq_file_write_char(seq_file, map->table[idx][0]);
			seq_file_write_char(seq_file, map->table[idx][1]);
		} else {
			seq_file_write_char(seq_file, *cmpstr);
		}
		cmpstr++;
	}
}


