/* 
 *   ccglue - cscope,ctags glue: Process ctags and cscope output to
 *        produce cross-reference table (for use with Vim CCTree plugin)
 *        and more ...
 *   Copyright (C) April, 2011,  Hari Rangarajan 
 *   Contact: hariranga@users.sourceforge.net

 *   "ccglue" is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with "ccglue".  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdio.h>
#include <string.h>
#include "seqfile.h"

struct seq_file {
	FILE *fp;
	uchar mode[2];
};

int seq_file_create (seq_file_t** seq_file, uchar* fname, uchar* mode) 
{
	if ((*seq_file = (seq_file_t *) malloc(sizeof(seq_file_t))) == NULL) {
		return RC_RUN_OUT_OF_MEMORY;
	}
	(*seq_file)->fp = fopen(fname, mode);
	if ((*seq_file)->fp == NULL) {
		free(*seq_file);
		return RC_FAILURE;
	}
	return RC_SUCCESS;
}

int seq_file_reader_create (seq_file_t** seq_file, uchar* fname) 
{
	return seq_file_create(seq_file, fname, "rb");
}

int seq_file_writer_create (seq_file_t** seq_file, uchar* fname) 
{
	return seq_file_create(seq_file, fname, "wb");
}

int seq_file_write_char (seq_file_t* seq_file, uchar c)
{
    return seq_file_write_buffer(seq_file, &c, sizeof(c));
}

int seq_file_write_string (seq_file_t* seq_file, uchar* str)
{
    return seq_file_write_buffer(seq_file, str, strlen(str));
}

int seq_file_write_buffer (seq_file_t* seq_file, uchar* buffer, size_t len)
{
    if (fwrite(buffer, sizeof(char), len, seq_file->fp) == len) {
        return RC_SUCCESS;
    } else {
        return RC_FAILURE;
    }
}

int seq_file_readline (seq_file_t* seq_file, uchar* line, int maxlen)
{
	int a;
	int idx = 0;
	while ((a = fgetc(seq_file->fp)) != EOF) {
		if (a == '\n') {
			break;
		} else if (idx >= maxlen) {
			/* TODO: we should handle this better */
			return RC_RUN_OUT_OF_MEMORY;
		}
		line[idx++] = a;
	}
	if (a == EOF) {
		return RC_FAILURE;
	}
	line[idx] = '\0';
	return RC_SUCCESS;
}

int seq_file_destroy (seq_file_t* seq_file)
{
	fclose(seq_file->fp);
	free(seq_file);
}

